package cologne.eck.note_pea;

import java.awt.Color;

/*
 * Peafactory - Production of Password Encryption Archives
 * Copyright (C) 2015  Axel von dem Bruch
 * 
 * This library is free software; you can redistribute it and/or modify it 
 * under the terms of the GNU General Public License as published 
 * by the Free Software Foundation; either version 2 of the License, 
 * or (at your option) any later version.
 * This library is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. 
 * See the GNU General Public License for more details.
 * See:  http://www.gnu.org/licenses/gpl-2.0.html
 * You should have received a copy of the GNU General Public License 
 * along with this library.
 */

/**
 * Dialog to get the password and start the editor pea.
 */


import java.awt.Insets;
import java.awt.Window;
import java.io.BufferedReader;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.IOException;
import java.io.InputStream;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Properties;

import javax.swing.JFileChooser;
import javax.swing.JOptionPane;
import javax.swing.SwingUtilities;
import javax.swing.UIManager;
import javax.swing.filechooser.FileNameExtensionFilter;
import javax.swing.text.DefaultStyledDocument;

import cologne.eck.all_peas.control.ContentUtilities;
import cologne.eck.all_peas.control.PeaControl;
import cologne.eck.all_peas.data.PeaProperties;
import cologne.eck.all_peas.files.FileComposer;
import cologne.eck.all_peas.files.FileModel;
import cologne.eck.all_peas.files.FileTypePanel;
import cologne.eck.all_peas.gui.CursorManager;
import cologne.eck.all_peas.gui.PeaFileChooser;
import cologne.eck.all_peas.gui.PswDialogView;

import cologne.eck.peafactory.crypto.CipherStuff;

import cologne.eck.tools.Converter;
import cologne.eck.tools.EntropyPool;
import cologne.eck.tools.ReadResources;
import cologne.eck.tools.UnexpectedValueException;
import cologne.eck.tools.PropertyHandler;


public final class EditorControl extends PeaControl {

	private static PswDialogView dialogView;
	private static EditorControl pswDialog;
	private static LockFrameEditor lockFrame; 
	private static String peaPropFileName = "editor.properties";
	
	private static final String VERSION = "1.2";
	private static final String YEAR_OF_PUBLICATION = "2019"; 
	private static final String WEBSITE = "/html/notebook_pea.html";
	private static final String SOURCE_LINK = "http://eck.cologne/peafactory/src/downloads/notebook/notebook-1.2-src.zip"; 
	

	private EditorControl() {
		
		pswDialog = this;		
		setDialog(pswDialog);
		dialogView = PswDialogView.getInstance();
	}
	
	public static EditorControl getInstance() {
		EditorControl dialogInstance = null;
		if (pswDialog == null) {			
			dialogInstance = new EditorControl();
		} else {
			// return null
		}
		return dialogInstance;
	}
	
	public static void main(String[] args) {
		
		PeaControl.setPeaPropFileName(peaPropFileName);
		new PropertyHandler().setAllPropValues();
		if(args.length > 0) {
			PeaProperties.setWorkingMode(args[0]);
		}

		SwingUtilities.invokeLater(new EventDispatchThread());
	
	}

	@Override
	public final LockFrameEditor getLockFrame() {
		return lockFrame;
	}

	@Override
	public void clearSecretValues() {		
		if (CipherStuff.getInstance().getSessionKeyCrypt() != null) {
			CipherStuff.getInstance().getSessionKeyCrypt().clearKeys();
		}
	}

	@Override
	public void startDecryption() {		

		String[] fileNames = null;
		
		if (PswDialogView.isInitializing() == false) {	

			// get files to decrypt:
			fileNames = PeaControl.getEncryptedFileTypePanel().getValidFileNames(true, true);
			
			if (fileNames == null) {
				PswDialogView.setMessage(PeaProperties.getBundle().getString("no_valid_file_selected"), true);
				PswDialogView.enableOkButton();
				return;
			}

			setSaltForEncryptedFiles(fileNames);

			// KDF:
			// derive the key from password and salt: 
			byte[] keyMaterial = getKeyMaterial();
			if (keyMaterial == null) {
				if (PeaProperties.getWorkingMode().equals("-t")) {
					new UnexpectedValueException("keyMaterial", "byte[]", "is null").printDescription();
				}
				PswDialogView.setMessage("Key derivation failed", true);
				PswDialogView.clearPassword();
				dialogView.pack();
				PswDialogView.enableOkButton();
				return;
			}
			
			// check the key:
			// read cipher text from first file
			int firstFileIndex = 0;
			int len = fileNames.length;
			// get first file in fileNames:
			for (int i = 0; i < len; i++) {
				if(new File(fileNames[i]).isFile()) {
					firstFileIndex = i;
					break;
				}
			}
			byte[] cipherText = ReadResources.readExternFile(fileNames[firstFileIndex]);
			if (cipherText == null) {
				if (PeaProperties.getWorkingMode().equals("-t")) {
					new UnexpectedValueException("cipherText", "byte[]", "is null for " + fileNames[firstFileIndex]).printDescription();
				} 		
			}

			// check password identifier
			String error = PeaControl.checkPassword(cipherText, keyMaterial);
			if (error != null) {
				// password failed: display error
				PswDialogView.setMessage(error, true);
				System.err.println("password failed");
				PswDialogView.enableOkButton();
				return;
			}		
			
			// do not decrypt, just store the key
			CipherStuff.getInstance().getSessionKeyCrypt().storeKey(keyMaterial);			
			// set the selected file names:
			PeaControl.setAvailableFileNames(new ArrayList<String>(Arrays.asList(fileNames)));
			
		} else { // initializing...
			// salt and nonce are set, file was prepared
			fileNames = new String[PeaControl.getAvailableFileNames().size()];
			fileNames = PeaControl.getAvailableFileNames().toArray(fileNames);
			System.out.println("Initialization");					
		}		
		
		// change background color
		PswDialogView.setUI(false);
		
		// lockFrame: 
		if (lockFrame == null) {
			new PropertyHandler().setSpecificPropValue();
			if (dialogView.getLocation() != null) {
				lockFrame = LockFrameEditor.getInstance(pswDialog, dialogView.getLocation());
			} else {
				lockFrame = LockFrameEditor.getInstance(pswDialog, PeaProperties.getFrameLocation());
			}
		}

		// iterate over file names: 
		int len = fileNames.length;
		for (int i = 0; i < len; i++) {
			// create a new tab for each file name: 
			lockFrame.createNewTab(fileNames[i]);			
		}

		PswDialogView.getView().setVisible(false);		

		lockFrame.setVisible(true);
		// set the ChangeListener when all tab were added:
		lockFrame.setChangeListener();
		
		EntropyPool.getInstance().stopCollection();
	}
	
	// not used here:
	@Override
	public void preComputeInThread() {}
	@Override
	public String[] getSelectedFileNames() {
		return ( (FileTypePanel)PeaProperties.getTypePanel()).getValidFileNames(true, false);
	}

	/**
	 * Other than File Lock PEA and Image Lock PEA, only
	 * one single file with extension .rtf can be initialized. 
	 * If a file with the same name exists, a warning and an 
	 * option to break is shown. If possible, the user is asked 
	 * to make the new file compatible to previous files 
	 * (using the same salt). 
	 */
	@Override
	public void initializeNewFiles() {		

	    PeaFileChooser chooser = new PeaFileChooser(PswDialogView.getView());

	    chooser.customizeFileChooser(
	    		PeaProperties.getBundle().getString("init_new_file"),
	    		PeaProperties.getBundle().getString("select_location_and_file_name"), 
	    		false);// no hidden files option
	    
		FileNameExtensionFilter txtfilter = new FileNameExtensionFilter(
                    "rich text format", "rtf");	
		
        chooser.setFileFilter(txtfilter);
        chooser.setFileSelectionMode(JFileChooser.FILES_ONLY);
	   // chooser.setCurrentDirectory(new File("/home/Documents"));
        Window win = null;
        if (PswDialogView.getView().isVisible() == true) {
        	win = PswDialogView.getView();
        } else {
        	win = lockFrame;
        }
	    int retrival = chooser.showOpenDialog(win);
	    if (retrival == JFileChooser.APPROVE_OPTION) {		    	
	    	
	    	File selectedFile = chooser.getSelectedFile();
	    	
	    	// append file extension:
	    	if ( ! selectedFile.getPath().endsWith(".rtf")){
	    		selectedFile = new File(selectedFile.getPath() + ".rtf");	
	    	}
	    	
	    	// Warning if file exists: 			    	
	    	if ( selectedFile.exists()){
	    		int n = JOptionPane.showConfirmDialog(dialogView,
	    				PeaProperties.getBundle().getString("overwrite_existing_file"), 
	    				PeaProperties.getBundle().getString("warning"), 
	    			JOptionPane.OK_CANCEL_OPTION,
	    			JOptionPane.WARNING_MESSAGE);
	    		if (n != JFileChooser.APPROVE_OPTION){ // cancel, close
	    			//return;
	    			System.exit(0);
	    		}
	    	}		    		    	
	    	try {
	    		PswDialogView.getView().setCursor(CursorManager.getWaitCursor());

	    		setSaltForNewFiles();
	    		// create an empty file:
	    		if (selectedFile.exists() == false) {
	    			boolean success = true;
	    			try {
	    				success = selectedFile.createNewFile();
	    			} catch (IOException e) {
	    				System.err.println("Creation of new empty file failed: " + selectedFile.getAbsolutePath());
	    				PswDialogView.setMessage( PeaProperties.getBundle().getString("read_write_failed") , true);
	    				JOptionPane.showMessageDialog(PswDialogView.getView(),
	    						PeaProperties.getBundle().getString("read_write_failed")
	    						+ "\n" + selectedFile.getAbsolutePath(),
	    						PeaProperties.getBundle().getString("error") + ": Creating new file",
	    						JOptionPane.ERROR_MESSAGE);			
	    				return;
	    			}
	    			if (success == false) {
	    				System.err.println("Creation of new empty file failed: " + selectedFile.getAbsolutePath());
	    				PswDialogView.setMessage( PeaProperties.getBundle().getString("read_write_failed") , true);
	    				JOptionPane.showMessageDialog(PswDialogView.getView(),
	    						PeaProperties.getBundle().getString("read_write_failed")
	    						+ "\n" + selectedFile.getAbsolutePath(),
	    						PeaProperties.getBundle().getString("error") + ": Creating new file",
	    						JOptionPane.ERROR_MESSAGE);			
	    				return;	    			
	    			}
	    		}

	    		ContentUtilities.saveContent(getKeyMaterial(), "\n".getBytes(PeaProperties.getCharset()), selectedFile.getPath());


	    		if (! PeaControl.getAvailableFileNames().contains(selectedFile.getPath())) {
	    			// delete the list
	    			PeaControl.setAvailableFileNames(new ArrayList<String>());
	    			// add selected file
	    			PeaControl.addAvailableFileName(selectedFile.getPath());
	    		}

	    		// store session key:				
	    		CipherStuff.getInstance().getSessionKeyCrypt().storeKey(getKeyMaterial());

	    	} finally {
	    		PswDialogView.getView().setCursor(CursorManager.getDefaultCursor());
	    	}

	    } else {
	    	System.err.println("file chooser canceled or closed");
	    	System.exit(0);
	    }  
	}

	@Override
	public void openEncryptedFiles() {

		FileComposer encryptedComposer = PeaControl.getEncryptedFileTypePanel().getFileComposer();
		FileModel encryptedModel = encryptedComposer.getFileModel();
		boolean added = encryptedComposer.addAction(PeaControl.getEncryptedFileTypePanel(),
				PeaProperties.getBundle().getString("select_files_to_encrypt"), 
				true, false);// hidden files, do not check listed files
		if (added == false) { // no file was added
			System.err.println("no file was added");
			return;
		}

		boolean validFileFound = encryptedModel.checkForValidFile();
		if (validFileFound == false) {
			//PswDialogView.setMessage(PeaProperties.getBundle().getString("no_valid_file_found"), true);
			PswDialogView.setMessage(PeaProperties.getBundle().getString("open_or_initialize"), true);
		} else {
			// clear errorMessageLabel
			PswDialogView.setMessage("   ", true);
		}		
	}
	@Override
	public String getVersion() {
		return VERSION;
	}
	@Override
	public String getYearOfPublication() {
		return YEAR_OF_PUBLICATION;
	}
	@Override
	public String getWebsite() {
		return WEBSITE;
	}
	@Override
	public String getSourceLink() {
		return SOURCE_LINK;
	}

	@Override
	public boolean checkFormat(File file, boolean easyCheck, boolean middleCheck, boolean expensiveCheck) {
		// TODO Auto-generated method stub
		//return false;
		String fileName = file.getAbsolutePath();
		if (easyCheck == true) {
			String extension = "";
			int i = fileName.lastIndexOf('.');
			if (i > 0) {
			    extension = fileName.substring(i+1);
			}
			if (! extension.toUpperCase().equalsIgnoreCase("RTF")) {
		    	if (PeaProperties.getWorkingMode().equals("-t")) {
					System.err.println(
							PeaProperties.getBundle().getString("unsuitable_rtf_file") 
							+ ": "  + fileName); 
					System.err.println("File extension is not .rtf");

		    	}
				return false;
			}
		}
		if (middleCheck == true) { // TODO test
			// check if file content starts with {\rtf1 in the first line
			BufferedReader brTest;
			String firstLine = "";
			try {
				brTest = new BufferedReader(new FileReader(file));
				firstLine = brTest .readLine();
			} catch (FileNotFoundException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			} catch (IOException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
		    if (! firstLine.contains("rtf1")) {
		    	if (PeaProperties.getWorkingMode().equals("-t")) {
				System.err.println(
						PeaProperties.getBundle().getString("unsuitable_rtf_file") 
						+ ": "  + fileName); 
				System.err.println("Unsuitable control sequenz: Could not find sequenz rtf1 in first line");
		    	}
		    	return false;
		    }
		}
		if (expensiveCheck == true) {
			Object obj = Converter.deserialize(ReadResources.readExternFile(fileName));		

			if (! (obj instanceof DefaultStyledDocument) ) { // wrong psw (malformed file) not all wrong files
		    	if (PeaProperties.getWorkingMode().equals("-t")) {
				System.err.println(
						PeaProperties.getBundle().getString("unsuitable_rtf_file") 
						+ ": "  + fileName);
				System.err.println("Unsuitable style of document");
		    	}
			}
		}
		return true;
	}

	@Override
	protected byte[] getPlainBytes() {
		return lockFrame.getPlainBytes();
		//return null;
	}

	@Override
	public String setPeaPropValues(Properties prop, InputStream inputStream) {
		String errorMessage = null;
		try {
			// Default font: ======================================
			String fontStyleString = prop.getProperty("font_style");
			if (fontStyleString != null) {
				try {
					NotebookSetting.setFontStyle(Integer.parseInt(fontStyleString));
				} catch (Exception e) {
					new UnexpectedValueException("prop", "Properties", "is not digit: " + prop.getProperty("font_style")).printDescription();
				}
			}			
			String fontFamily = prop.getProperty("font_family");
			if (fontFamily != null) {
				NotebookSetting.setFontFamily(fontFamily);
			}
			String fontSizeString = prop.getProperty("text_font_size");
			if (fontSizeString != null) {
				try {
					NotebookSetting.setFontSize(Integer.parseInt(fontSizeString));
				} catch (Exception e) {
					new UnexpectedValueException("prop", "Properties", "is not digit: " + prop.getProperty("text_font_size")).printDescription();
				}
			}
			//
			int r = 0, g = 0, b = 0;
			String rString = prop.getProperty("text_background_color_r");
			if (rString != null) {
				try {
					r = Integer.parseInt(rString);
				} catch (Exception e) {
					r = NotebookSetting.getTextBackground().getRed();
					new UnexpectedValueException("rString", "String", "is not digit: " + rString).printDescription();
				}
			} else {
				r = NotebookSetting.getTextBackground().getRed();
			}
			String gString = prop.getProperty("text_background_color_g");
			if (gString != null) {
				try {
					g = Integer.parseInt(gString);
				} catch (Exception e) {
					g = NotebookSetting.getTextBackground().getGreen();
					new UnexpectedValueException("gString", "String", "is not digit: " + gString).printDescription();
				}
			} else {
				g = NotebookSetting.getTextBackground().getGreen();
			}
			String bString = prop.getProperty("text_background_color_b");
			if (bString != null) {
				try {
					b = Integer.parseInt(bString);
				} catch (Exception e) {
					b = NotebookSetting.getTextBackground().getBlue();
					new UnexpectedValueException("bString", "String", "is not digit: " + bString).printDescription();
				}
			} else {
				b = NotebookSetting.getTextBackground().getBlue();
			}
			NotebookSetting.setTextBackground(new Color(r, g, b));
			
			
			String rFontString = prop.getProperty("font_color_r");
			if (rFontString != null) {
				try {
					r = Integer.parseInt(rFontString);
				} catch (Exception e) {
					r = NotebookSetting.getFontColor().getRed();
					new UnexpectedValueException("rFontString", "String", "is not digit: " + rFontString).printDescription();
				}
			} else {
				r = NotebookSetting.getFontColor().getRed();
			}
			String gFontString = prop.getProperty("font_color_g");
			if (gFontString != null) {
				try {
					g = Integer.parseInt(gFontString);
				} catch (Exception e) {
					g = NotebookSetting.getFontColor().getGreen();
					new UnexpectedValueException("gFontString", "String", "is not digit: " + gFontString).printDescription();
				}
			} else {
				g = NotebookSetting.getFontColor().getGreen();
			}
			String bFontString = prop.getProperty("font_color_b");
			if (bFontString != null) {
				try {
					b = Integer.parseInt(bFontString);
				} catch (Exception e) {
					b = NotebookSetting.getFontColor().getBlue();
					new UnexpectedValueException("bFontString", "String", "is not digit: " + bFontString).printDescription();
				}
			} else {
				b = NotebookSetting.getFontColor().getBlue();
			}
			NotebookSetting.setFontColor(new Color(r, g, b));

					
		} catch ( Exception e){
			System.err.println(e + ", " + e.getMessage());
			new UnexpectedValueException("","","set pea properties").printDescription();
			errorMessage = e.toString() + ", " + e.getLocalizedMessage();
		}
		return errorMessage;

	}

	@Override
	public String updatePeaPropValues(Properties prop) {//, InputStream inputStream) {
		String errorMessage = null;
		try {
			prop.setProperty("font_style", "" + NotebookSetting.getFontStyle());
			prop.setProperty("font_family", NotebookSetting.getFontFamily());
			prop.setProperty("text_font_size", "" + NotebookSetting.getFontSize());

			prop.setProperty("text_background_color_r","" + NotebookSetting.getTextBackground().getRed());
			prop.setProperty("text_background_color_g","" + NotebookSetting.getTextBackground().getGreen());
			prop.setProperty("text_background_color_b","" + NotebookSetting.getTextBackground().getBlue());

			prop.setProperty("font_color_r", "" + NotebookSetting.getFontColor().getRed());
			prop.setProperty("font_color_g", "" + NotebookSetting.getFontColor().getGreen());
			prop.setProperty("font_color_b", "" + NotebookSetting.getFontColor().getBlue());
			
					
		} catch ( Exception e){
			System.err.println(e + ", " + e.getMessage());
			errorMessage = e.toString() + ", " + e.getLocalizedMessage();
		}
		return errorMessage;	
	}
	
	static class  EventDispatchThread implements Runnable{

		@Override
		public void run() {
			PswDialogView.setUI(true);
			// customize button margin
			UIManager.put("Button.contentMargins", new Insets( 5,9,5,9) );
					
			initializeVariables();		
			// settings:
			PeaProperties.setFileType("text file");		
			pswDialog = EditorControl.getInstance();		
			setDialog(pswDialog);			
			EditorControl.dialogView.setVisible(true);
			LockFrameEditor.setDocChangeUnsaved(false);			
		}		
	}
}	